@@ -15,7 +15,7 @@ from typing import Any
 
 from xpra.common import noop
 from xpra.util.objects import typedict
-from xpra.util.str_fn import csv, Ellipsizer, repr_ellipsized, pver, strtobytes, bytestostr, hexstr, memoryview_to_bytes
+from xpra.util.str_fn import csv, Ellipsizer, repr_ellipsized, pver, bytestostr, hexstr, memoryview_to_bytes
 from xpra.util.env import envint, envbool, osexpand, first_time, IgnoreWarningsContext, ignorewarnings
 from xpra.util.child_reaper import getChildReaper
 from xpra.os_util import gi_import, WIN32, OSX, POSIX
@@ -29,17 +29,15 @@ from xpra.gtk.cursors import cursor_types, get_default_cursor
 from xpra.gtk.util import get_default_root_window, get_root_size, GRAB_STATUS_STRING, init_display_source
 from xpra.gtk.window import GDKWindow
 from xpra.gtk.info import get_monitors_info, get_screen_sizes
-from xpra.gtk.widget import scaled_image, label, color_parse, FILE_CHOOSER_NATIVE
+from xpra.gtk.widget import scaled_image, label, FILE_CHOOSER_NATIVE
 from xpra.gtk.pixbuf import get_icon_pixbuf, get_pixbuf_from_data
 from xpra.gtk.versions import get_gtk_version_info
 from xpra.exit_codes import ExitCode, ExitValue
 from xpra.gtk.gobject import no_arg_signal
 from xpra.gtk.css_overrides import inject_css_overrides
 from xpra.client.gui.ui_client_base import UIXpraClient
-from xpra.client.gui.window_border import WindowBorder
 from xpra.client.base.gobject import GObjectXpraClient
 from xpra.client.gtk3.keyboard_helper import GTKKeyboardHelper
-from xpra.client.mixins.windows import WindowClient
 from xpra.platform.gui import force_focus
 from xpra.platform.gui import (
     get_window_frame_sizes, get_window_frame_size,
@@ -94,17 +92,6 @@ def get_group_ref(metadata: typedict) -> str:
     return ""
 
 
-# pylint: disable=import-outside-toplevel
-def show_border_help() -> None:
-    if not first_time("border-help"):
-        return
-    log.info(" border format: color[,size][:off]")
-    log.info("  eg: red,10")
-    log.info("  eg: ,5")
-    log.info("  eg: auto,5")
-    log.info("  eg: blue")
-
-
 # noinspection PyMethodMayBeStatic
 class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     __gsignals__ = {}
@@ -128,7 +115,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self.start_new_command = None
         self.server_commands = None
         self.keyboard_helper_class = GTKKeyboardHelper
-        self.border: WindowBorder | None = None
         self.data_send_requests = {}
         # clipboard bits:
         self.clipboard_notification_timer = 0
@@ -141,7 +127,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self.gl_max_viewport_dims = 0, 0
         self.gl_texture_size_limit = 0
         self._cursors = weakref.WeakKeyDictionary()
-        self.border_str = "no"
         # frame request hidden window:
         self.frame_request_window = None
         # group leader bits:
@@ -160,7 +145,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def init(self, opts) -> None:
         GObjectXpraClient.init(self, opts)
         UIXpraClient.init(self, opts)
-        self.border_str = opts.border
 
     def setup_frame_request_windows(self) -> None:
         # query the window manager to get the frame size:
@@ -168,19 +152,19 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         from xpra.x11.bindings.send_wm import send_wm_request_frame_extents
         self.frame_request_window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
         self.frame_request_window.set_title("Xpra-FRAME_EXTENTS")
-        root = self.get_root_window()
         self.frame_request_window.realize()
+        win = self.frame_request_window.get_window()
+        xid = win.get_xid()
+        framelog("setup_frame_request_windows() window=%#x", xid)
         with xsync:
-            win = self.frame_request_window.get_window()
-            xid = win.get_xid()
-            framelog("setup_frame_request_windows() window=%#x", xid)
-            send_wm_request_frame_extents(root.get_xid(), xid)
+            root_xid = self.get_root_xid()
+            send_wm_request_frame_extents(root_xid, xid)
 
     def run(self) -> ExitValue:
         log(f"run() HAS_X11_BINDINGS={HAS_X11_BINDINGS}")
-        # call this once early:
         from xpra.client.gui import features
         if features.windows:
+            # call this once early:
             ignorewarnings(self.get_mouse_position)
             if HAS_X11_BINDINGS:
                 self.setup_frame_request_windows()
@@ -432,61 +416,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         authlog("showing challenge prompt dialog")
         dialog.show()
 
-    def setup_connection(self, conn):
-        conn = super().setup_connection(conn)
-        # now that we have display_desc, parse the border again:
-        self.parse_border(False)
-        return conn
-
-    def parse_border(self, warn=True) -> None:
-        enabled = not self.border_str.endswith(":off")
-        parts = [x.strip() for x in self.border_str.replace(":off", "").split(",")]
-        color_str = parts[0]
-        if color_str.lower() in ("none", "no", "off", "0"):
-            return
-        if color_str.lower() == "help":
-            show_border_help()
-            return
-        color_str = color_str.replace(":off", "")
-        if color_str in ("auto", ""):
-            from hashlib import sha256
-            m = sha256()
-            endpoint = self.display_desc.get("display_name")
-            if endpoint:
-                m.update(strtobytes(endpoint))
-            color_str = "#%s" % m.hexdigest()[:6]
-            log(f"border color derived from {endpoint}: {color_str}")
-        try:
-            color = color_parse(color_str)
-            assert color is not None
-        except Exception as e:
-            if warn:
-                log.warn(f"Warning: invalid border color specified '{color_str!r}'")
-                if str(e):
-                    log.warn(" %s", e)
-                show_border_help()
-            color = color_parse("red")
-        alpha = 0.6
-        size = 4
-        if len(parts) == 2:
-            size_str = parts[1]
-            try:
-                size = int(size_str)
-            except Exception as e:
-                if warn:
-                    log.warn(f"Warning: invalid border size specified {size_str!r}")
-                    log.warn(f" {e}")
-                    show_border_help()
-            if size <= 0:
-                log(f"border size is {size}, disabling it")
-                return
-            if size >= 45:
-                log.warn(f"Warning: border size is too large: {size}, clipping it")
-                size = 45
-        self.border = WindowBorder(enabled, color.red / 65536.0, color.green / 65536.0, color.blue / 65536.0, alpha,
-                                   size)
-        log("parse_border(%s)=%s", self.border_str, self.border)
-
     def show_server_commands(self, *_args) -> None:
         if not self.server_commands_info:
             log.warn("Warning: cannot show server commands")
@@ -562,7 +491,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         if fad:
             GLib.idle_add(fad.transfer_progress_update, send, transfer_id, elapsed, position, total, error)
 
-    def accept_data(self, send_id, dtype: str, url: str, printit: bool, openit: bool) -> tuple:
+    def accept_data(self, send_id, dtype: str, url: str, printit: bool, openit: bool) -> tuple[bool, bool, bool]:
         # check if we have accepted this file via the GUI:
         r = self.data_send_requests.pop(send_id, None)
         if not r:
@@ -580,9 +509,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             if eurl != url:
                 filelog.warn(" expected url '%s',", s(eurl))
                 filelog.warn("  but got url '%s'", s(url))
-            return None
+            return False, False, False
         # return the printit and openit flag we got from the UI:
-        return r[2], r[3]
+        ui_printit = bool(r[2])
+        ui_openit = bool(r[3])
+        return True, ui_printit, ui_openit
 
     def file_size_warning(self, action: str, location: str, basefilename: str, filesize: int, limit: int) -> None:
         if self.file_size_dialog:
@@ -611,7 +542,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.file_size_dialog = None
             fsd.close()
 
-    def download_server_log(self, callback: Callable = noop) -> None:
+    def download_server_log(self, callback: Callable[[str, int], None] = noop) -> None:
         filename = "${XPRA_SERVER_LOG}"
         self.file_request_callback[filename] = callback
         self.send_request_file(filename, self.open_files)
@@ -743,7 +674,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         # gives the server time to send an info response..
         # (by the time the user clicks on copy, it should have arrived, we hope!)
 
-        def got_server_log(filename: str, filesize: int):
+        def got_server_log(filename: str, filesize: int) -> None:
             log(f"got_server_log({filename!r}, {filesize})")
             filedata = load_binary_file(filename)
             self.bug_report.set_server_log_data(filedata)
@@ -762,12 +693,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def request_frame_extents(self, window) -> None:
         from xpra.x11.bindings.send_wm import send_wm_request_frame_extents
         from xpra.gtk.error import xsync
-        root = self.get_root_window()
+        win = window.get_window()
+        xid = win.get_xid()
+        framelog(f"request_frame_extents({window}) xid={xid:x}")
         with xsync:
-            win = window.get_window()
-            xid = win.get_xid()
-            framelog(f"request_frame_extents({window}) xid={xid:x}")
-            send_wm_request_frame_extents(root.get_xid(), xid)
+            root_xid = self.get_root_xid()
+            send_wm_request_frame_extents(root_xid, xid)
 
     def get_frame_extents(self, window):
         # try native platform code first:
@@ -840,6 +771,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return self._add_statusicon_tray(TrayClient.get_tray_classes(self))
 
     def get_system_tray_classes(self) -> list[type]:
+        from xpra.client.mixins.windows import WindowClient
         return self._add_statusicon_tray(WindowClient.get_system_tray_classes(self))
 
     def supports_system_tray(self) -> bool:
@@ -849,6 +781,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def get_root_window(self):
         return get_default_root_window()
 
+    def get_root_xid(self) -> int:
+        assert HAS_X11_BINDINGS
+        from xpra.x11.bindings.window import X11WindowBindings
+        return X11WindowBindings().get_root_xid()
+
     def get_root_size(self):
         return get_root_size()
 
@@ -1115,11 +1052,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         confine_to = None
         cursor = None
         etime = Gtk.get_current_event_time()
-        r = Gdk.pointer_grab(window.get_window(), True, event_mask, confine_to, cursor, etime)
-        grablog("pointer_grab(..)=%s", GRAB_STATUS_STRING.get(r, r))
-        # also grab the keyboard so the user won't Alt-Tab away:
-        r = Gdk.keyboard_grab(window.get_window(), False, etime)
-        grablog("keyboard_grab(..)=%s", GRAB_STATUS_STRING.get(r, r))
+        with IgnoreWarningsContext():
+            r = Gdk.pointer_grab(window.get_window(), True, event_mask, confine_to, cursor, etime)
+            grablog("pointer_grab(..)=%s", GRAB_STATUS_STRING.get(r, r))
+            # also grab the keyboard so the user won't Alt-Tab away:
+            r = Gdk.keyboard_grab(window.get_window(), False, etime)
+            grablog("keyboard_grab(..)=%s", GRAB_STATUS_STRING.get(r, r))
         self._window_with_grab = wid
 
     def window_ungrab(self) -> None:
@@ -1137,8 +1075,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             gdkwindow = window.get_window()
         if gdkwindow is None:
             gdkwindow = self.get_root_window()
-        log(f"window_bell(..) gdkwindow={gdkwindow}")
-        if not system_bell(gdkwindow, device, percent, pitch, duration, bell_class, bell_id, bell_name):
+        xid = 0
+        if hasattr(gdkwindow, "get_xid"):
+            xid = gdkwindow.get_xid()
+        log(f"window_bell(..) {gdkwindow=}, {xid=}")
+        if not system_bell(xid, device, percent, pitch, duration, bell_class, bell_id, bell_name):
             # fallback to simple beep:
             Gdk.beep()
 
@@ -1219,12 +1160,13 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         parts = enable_opengl.split(":", 1)
         enable_option = parts[0].lower()  # ie: "on"
         opengllog(f"init_opengl: enable_option={enable_option}")
-        if enable_option in ("probe-failed", "probe-error", "probe-crash"):
+        if enable_option in ("probe-failed", "probe-error", "probe-crash", "probe-warning", "probe-disabled"):
             msg = enable_option.replace("-", " ")
             if len(parts) > 1 and any(len(x) for x in parts[1:]):
                 msg += ": %s" % csv(parts[1:])
             self.opengl_props["info"] = "disabled, %s" % msg
-            self.opengl_setup_failure(body=msg)
+            if enable_option != "probe-disabled":
+                self.opengl_setup_failure(body=msg)
             return
         if enable_option in FALSE_OPTIONS:
             self.opengl_props["info"] = "disabled by configuration"
@@ -1267,9 +1209,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 opengllog.warn(" %s", warning)
             self.opengl_props["info"] = "enabled despite: %s" % csv(warnings)
         try:
-            opengllog("init_opengl: going to import xpra.client.gl")
-            __import__("xpra.client.gl", {}, {}, [])
-            from xpra.client.gl.window import get_gl_client_window_module, test_gl_client_window
+            opengllog("init_opengl: going to import xpra.opengl")
+            __import__("xpra.opengl", {}, {}, [])
+            from xpra.opengl.window import get_gl_client_window_module, test_gl_client_window
             self.opengl_props, gl_client_window_module = get_gl_client_window_module(enable_opengl)
             if not gl_client_window_module:
                 opengllog.warn("Warning: no OpenGL backend module found")
@@ -1297,7 +1239,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     opengllog.warn("Warning: OpenGL is disabled:")
                 opengllog.warn(" %s", warning)
 
-            from xpra.client.gl.check import MIN_SIZE
+            from xpra.opengl.check import MIN_SIZE
             if min(self.gl_max_viewport_dims) < MIN_SIZE:
                 glwarn("the maximum viewport size is too low: %s" % (self.gl_max_viewport_dims,))
             if self.gl_texture_size_limit < MIN_SIZE:
@@ -1342,7 +1284,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     return
                 opengllog(f"OpenGL test rendering succeeded: {draw_result}")
             if self.opengl_enabled:
-                opengllog.info(f"OpenGL enabled on {driver_info!r}")
+                glvstr = ".".join(str(v) for v in self.opengl_props.get("opengl", ()))
+                opengllog.info(f"OpenGL {glvstr} enabled on {driver_info!r}")
+                module = self.opengl_props.get("module", "unknown")
+                backend = self.opengl_props.get("backend", "unknown")
+                opengllog.info(f" using {module} {backend} backend")
+                opengllog.info(" zerocopy is %s", ["not available", "available"][self.opengl_props.get("zerocopy", 0)])
                 # don't try to handle video dimensions bigger than this:
                 mvs = min(8192, self.gl_texture_size_limit)
                 self.video_max_size = (mvs, mvs)
@@ -1472,6 +1419,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def destroy_window(self, wid: int, window) -> None:
         # override so we can cleanup the group-leader if needed,
+        from xpra.client.mixins.windows import WindowClient
         WindowClient.destroy_window(self, wid, window)
         group_leader = window.group_leader
         if group_leader is None or not self._group_leader_wids:
