@@ -6,7 +6,7 @@
 import os
 import re
 from typing import Any
-from collections.abc import Sequence, Callable
+from collections.abc import Sequence, Callable, Iterable
 
 from xpra.util.objects import typedict, reverse_dict
 from xpra.util.str_fn import Ellipsizer, repr_ellipsized, bytestostr
@@ -81,6 +81,9 @@ CLIPBOARD_DIRECTION_NAME_TO_LABEL = reverse_dict(CLIPBOARD_DIRECTION_LABEL_TO_NA
 
 SERVER_NOT_SUPPORTED = "Not supported by the server"
 
+# 'auto' is recorded as '' unfortunately:
+GENERIC_ENCODINGS = ("", "stream", "grayscale")
+
 
 class GTKTrayMenu(MenuHelper):
 
@@ -475,7 +478,7 @@ class GTKTrayMenu(MenuHelper):
             kh = self.client.keyboard_helper
             if not kh:
                 text = "Keyboard support is not loaded"
-            elif kh.keyboard_sync:
+            elif kh.sync:
                 text = "Disable keyboard synchronization " + \
                        "(prevents spurious key repeats on high latency connections)"
             else:
@@ -500,8 +503,8 @@ class GTKTrayMenu(MenuHelper):
             if can_set_sync:
                 self.keyboard_sync_menuitem.connect("toggled", keyboard_sync_toggled)
             if kh:
-                log("set_keyboard_sync_menuitem%s enabled=%s", args, kh.keyboard_sync)
-            self.keyboard_sync_menuitem.set_active(kh and bool(kh.keyboard_sync))
+                log("set_keyboard_sync_menuitem%s enabled=%s", args, kh.sync)
+            self.keyboard_sync_menuitem.set_active(kh and bool(kh.sync))
             set_keyboard_sync_tooltip()
 
         self.after_handshake(set_keyboard_sync_menuitem)
@@ -810,7 +813,7 @@ class GTKTrayMenu(MenuHelper):
     def set_qualitymenu(self, *_args) -> None:
         if self.quality:
             enc = self.client.encoding
-            with_quality = enc in self.client.server_encodings_with_quality or self.client.encoding == "auto"
+            with_quality = enc in self.client.server_encodings_with_quality or enc in GENERIC_ENCODINGS
             can_use = with_quality and not self.client.mmap_enabled
             set_sensitive(self.quality, can_use)
             if self.client.mmap_enabled:
@@ -862,7 +865,7 @@ class GTKTrayMenu(MenuHelper):
     def set_speedmenu(self, *_args) -> None:
         if self.speed:
             enc = self.client.encoding
-            with_speed = enc in self.client.server_encodings_with_speed or self.client.encoding == "auto"
+            with_speed = enc in self.client.server_encodings_with_speed or enc in GENERIC_ENCODINGS
             set_sensitive(self.speed, with_speed and not self.client.mmap_enabled)
             if self.client.mmap_enabled:
                 self.speed.set_tooltip_text("Quality is always 100% with mmap")
@@ -1175,7 +1178,7 @@ class GTKTrayMenu(MenuHelper):
         self.layout_submenu = Gtk.Menu()
         keyboard.set_submenu(self.layout_submenu)
 
-        def kbitem(title, layout, variant, active=False) -> Gtk.CheckMenuItem:
+        def kbitem(title: str, layout: str, variant: str, active=False) -> Gtk.CheckMenuItem:
             def set_layout(item) -> None:
                 """ this callback updates the client (and server) if needed """
                 ensure_item_selected(self.layout_submenu, item)
@@ -1203,16 +1206,16 @@ class GTKTrayMenu(MenuHelper):
             l.keyboard_variant = variant
             return l
 
-        def keysort(key):
+        def keysort(key) -> str:
             c, l = key
             return c.lower() + l.lower()
 
-        def variants_submenu(layout, variants) -> None:
+        def variants_submenu(layout: str, variants: Iterable[str]) -> None:
             # just show all the variants to choose from this layout
             default_layout = kbitem(f"{layout} - Default", layout, "", True)
             self.layout_submenu.append(default_layout)
-            for v in variants:
-                self.layout_submenu.append(kbitem(f"{layout} - {v}", layout, v))
+            for variant in variants:
+                self.layout_submenu.append(kbitem(f"{layout} - {variant}", layout, variant))
 
         kh = self.client.keyboard_helper
         layout, layouts, variant, variants, _ = kh.get_layout_spec()
@@ -1226,7 +1229,7 @@ class GTKTrayMenu(MenuHelper):
 
             # log after removing dupes:
 
-            def uniq(seq):
+            def uniq(seq: Iterable) -> list:
                 seen = set()
                 return [x for x in seq if not (x in seen or seen.add(x))]
 
@@ -1273,12 +1276,12 @@ class GTKTrayMenu(MenuHelper):
                     variant_submenu = Gtk.Menu()
                     variant.set_submenu(variant_submenu)
                     self.layout_submenu.append(variant)
-                    variant_submenu.append(kbitem(f"{layout} - Default", layout, None))
+                    variant_submenu.append(kbitem(f"{layout} - Default", layout, ""))
                     for v in variants:
                         variant_submenu.append(kbitem(f"{layout} - {v}", layout, v))
                 else:
                     # no variants:
-                    self.layout_submenu.append(kbitem(name, layout, None))
+                    self.layout_submenu.append(kbitem(name, layout, ""))
         self.after_handshake(set_sensitive, keyboard, True)
         return keyboard
 
