@@ -14,7 +14,7 @@ from http.server import BaseHTTPRequestHandler
 from typing import Any
 from collections.abc import Iterable, Callable
 
-from xpra.common import DEFAULT_XDG_DATA_DIRS, FULL_INFO
+from xpra.common import FULL_INFO
 from xpra.net.common import HttpResponse
 from xpra.net.http.common import EXTENSION_TO_MIMETYPE
 from xpra.net.http.directory_listing import list_directory
@@ -94,12 +94,6 @@ def translate_path(path: str, web_root: str = "/usr/share/xpra/www") -> str:
     words = path.split('/')
     words = list(filter(None, words))
     path = web_root
-    xdg_data_dirs = os.environ.get("XDG_DATA_DIRS", DEFAULT_XDG_DATA_DIRS)
-    www_dir_options = [web_root] + [os.path.join(x, "xpra", "www") for x in xdg_data_dirs.split(":")]
-    for p in www_dir_options:
-        if os.path.exists(p) and os.path.isdir(p):
-            path = p
-            break
     for word in words:
         word = os.path.splitdrive(word)[1]
         word = os.path.split(word)[1]
@@ -196,6 +190,7 @@ def load_path(accept_encoding: list[str], path: str) -> tuple[int, dict[str, Any
         return 200, extra_headers, content
 
 
+# noinspection PyPep8Naming
 class HTTPRequestHandler(BaseHTTPRequestHandler):
     """
     Xpra's builtin HTTP server.
@@ -410,11 +405,12 @@ class HTTPRequestHandler(BaseHTTPRequestHandler):
         except OSError as e:
             self.close_connection = True
             log("send_head()", exc_info=True)
-            log.error("Error sending '%s':", path)
-            emsg = str(e)
-            if emsg.endswith(f": '{path}'"):
-                log.error(" %s", emsg.rsplit(":", 1)[0])
-            else:
-                log.estr(e)
-            self.send_error(404, "File not found")
+            if not isinstance(e, ConnectionResetError):
+                log.error("Error sending '%s':", path)
+                emsg = str(e)
+                if emsg.endswith(f": '{path}'"):
+                    log.error(" %s", emsg.rsplit(":", 1)[0])
+                else:
+                    log.estr(e)
+                self.send_error(404, "File not found")
             return b""
