@@ -5,14 +5,15 @@
 
 import os
 import struct
-from collections.abc import ByteString, Callable
+from collections.abc import Callable
 
+from xpra.common import SizedBuffer
 from xpra.net.websockets.mask import hybi_mask
 from xpra.net.websockets.header import encode_hybi_header, decode_hybi, close_packet
 from xpra.net.websockets.common import OPCODE, OPCODE_STR
 from xpra.net.protocol.socket_handler import SocketProtocol
 from xpra.util.env import envbool, first_time
-from xpra.util.str_fn import hexstr, memoryview_to_bytes
+from xpra.util.str_fn import hexstr, memoryview_to_bytes, Ellipsizer
 from xpra.log import Logger
 
 log = Logger("websocket")
@@ -25,8 +26,8 @@ class WebSocketProtocol(SocketProtocol):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.ws_data: ByteString = b""
-        self.ws_payload: list[ByteString] = []
+        self.ws_data: bytes = b""
+        self.ws_payload: list[SizedBuffer] = []
         self.ws_payload_opcode: int = 0
         self.ws_mask: bool = MASK
         self._process_read = self.parse_ws_frame
@@ -48,7 +49,7 @@ class WebSocketProtocol(SocketProtocol):
         data = close_packet(code, reason)
         self.flush_then_close(None, data)
 
-    def make_wsframe_header(self, packet_type, items) -> ByteString:
+    def make_wsframe_header(self, packet_type: str, items: list[SizedBuffer]) -> bytes:
         payload_len = sum(len(item) for item in items)
         header = encode_hybi_header(OPCODE.BINARY, payload_len, self.ws_mask)
         log("make_wsframe_header(%s, %i items) %i bytes, ws_mask=%s, header=0x%s (%i bytes)",
@@ -61,7 +62,7 @@ class WebSocketProtocol(SocketProtocol):
             return header + mask
         return header
 
-    def parse_ws_frame(self, buf: ByteString) -> None:
+    def parse_ws_frame(self, buf: SizedBuffer) -> None:
         if not buf:
             self._read_queue_put(buf)
             return
@@ -76,7 +77,7 @@ class WebSocketProtocol(SocketProtocol):
         while ws_data and not self._closed:
             parsed = decode_hybi(ws_data)
             if parsed is None:
-                log("parse_ws_frame(%i bytes) not enough data: %r", len(ws_data), ws_data)
+                log("parse_ws_frame(%i bytes) not enough data: %r", len(ws_data), Ellipsizer(ws_data))
                 # not enough data to get a full websocket frame,
                 # save it for later:
                 self.ws_data = ws_data
@@ -127,17 +128,18 @@ class WebSocketProtocol(SocketProtocol):
                 log.warn("Warning unhandled websocket opcode '%s'", OPCODE_STR.get(opcode, f"{opcode:x}"))
                 log("payload=%r", payload)
 
-    def _process_ws_ping(self, payload: ByteString) -> None:
+    def _process_ws_ping(self, payload: SizedBuffer) -> None:
         log("_process_ws_ping(%r)", payload)
         item = encode_hybi_header(OPCODE.PONG, len(payload)) + memoryview_to_bytes(payload)
         items = (item,)
         with self._write_lock:
             self.raw_write("ws-ping", items)
 
-    def _process_ws_pong(self, payload: ByteString) -> None:
+    @staticmethod
+    def _process_ws_pong(payload: SizedBuffer) -> None:
         log("_process_ws_pong(%r)", payload)
 
-    def _process_ws_close(self, payload: ByteString) -> None:
+    def _process_ws_close(self, payload: SizedBuffer) -> None:
         log("_process_ws_close(%r)", payload)
         if len(payload) < 2:
             self._connection_lost("unknown reason")
